<?php
/**
 * NLSClientScript class v1.2
 *
 * @author nlac <nlacsoft@gmail.com>
 * @link http://nlacsoft.net
 * @license http://nlacsoft.net
 * 
 * History:
 * 1.2
 *  - fixed js error when app not in YII_DEBUG mode
 * 1.1
 *  - hash key generated on server side
 *  - two hash key mode: PATH and CONTENT
 *  - shortened client-side code
 * 1.0
 *  - base version
 * 
 */

/**
 * CClientScript extension for handling managed lazy loading of javascript files
 */
class NLSClientScript extends CClientScript
{


/**
 * This value determines how the hash key should be composed for a resource file.
 * Possible values:
 *   CONTENT: a bit slower but safe
 *   PATH: 
 *     - the key is composed from the file name
 *     - there may be problem if the names of two different resource files used in the same view are equal
 *      ( eg. two different "main.js" in the same view)
 *      this case causes that the loading of the 2nd one will be denied -> rename the file or use CONTENT mode
 * 
 *   CONTENT:
 *     - the key is computed from the file content
 *     - a bit more processing on the server side
 * 
 * 
 * 
 **/
	public $hashMode = 'PATH';//PATH|CONTENT




/**
 * Composing the key from the file content
 */
	private static function getHashKey($f) {
		//return $_SERVER['DOCUMENT_ROOT'] . $f;
		//return preg_replace('/\/assets\/[\w]+/','/ASSET',$f);
		
		if (Yii::app()->clientScript->hashMode == 'PATH') {
			//Composing thekey from the file name
			
			//If the file name contains "jquery" then chunk the path 
			//(assuming that the file name is unique over the project)
			if (stristr($f,'jquery')!==false) {
				preg_match('/[^\\/\\\\]+\.(js|css)$/', $f, $matches);
				return @$matches[0];
			}
			return '';
		}
		
		//Composing the key from the file content
		//$fullPath = $_SERVER['DOCUMENT_ROOT'] . preg_replace('/[\\/\\\\]/',DIRECTORY_SEPARATOR,$f);
		$fullPath = $_SERVER['DOCUMENT_ROOT'] . $f;
		$c = @file_get_contents( $fullPath );
		if (!$c) {
			Yii::log('no resource found on path ' . $fullPath, 'warning');
			return false;
		}
		//Yii::log('fullpath: ' . $fullPath . ', dirsep='.DIRECTORY_SEPARATOR, 'warning');
		return substr(md5( $c ),0,8);
	}

	private static function scriptFile($f) {
		//if (typeof jQuery != "undefined" && jQuery.yii && jQuery.yii.lazyLoad)
		$hk = self::getHashKey($f);
		return $hk!==false ? CHtml::script('$.yii.lazyLoad("' . $f . '","' . $hk . '");') : '';
	}
	
	private static function cssFile($f, $media) {
		$hk = self::getHashKey($f);
		return $hk!==false ? CHtml::script('$.yii.lazyLoad("' . $f . '","' . $hk . '",{media:"' . $media . '"});') : '';
	}

/**
 * Generates usual linking code of base scripts
 * @return the html linking code
 */
	protected static function _renderBaseScriptFiles() {
		if (!Yii::app()->request->isAjaxRequest) {

			$cs = Yii::app()->getClientScript();
			
			//Deploying system assets statically
			$assetsPath = Yii::app()->getAssetManager()->publish( Yii::getPathOfAlias( 'system.web.js'), false, -1, false );
			
			$scripts = array(
				$assetsPath . (defined('YII_DEBUG') && YII_DEBUG ? '/source/jquery.js' : '/source/jquery.min.js'),
				$assetsPath . '/source/jquery.yii.js'
			);
			
			$html = "";
			$scriptList = "[";
			foreach($scripts as $s) {
				$html .= CHtml::scriptFile($s) . "\n";//! linking by the static method
				$scriptList .= '{path:"' . $s . '",hk:"' . self::getHashKey($s) . '"},';
			}
			$scriptList = rtrim($scriptList, ',') . ']';

			$html .= CHtml::script('
(function($){

$.extend($.yii, {
	_rescache : {},

	is_a : function(obj, _class) {
		return (typeof obj == "object" && obj && obj.constructor == _class);
	},
	
	toCache : function(f,hk) {
		if (this.is_a(f,Array)) {
			for(var i=0; i<f.length; i++)
				this.toCache(f[i].path,f[i].hk);
			return;
		}
		if (!hk) hk=f;
		this._rescache[hk] = f;
	},

	lazyLoad : function(f, hk, opt) {
		if (this.is_a(f,Array)) {
			for(var i=0; i<f.length; i++)
				this.lazyLoad(f[i].path, f[i].hk, opt);
			return;
		}
		
		if (!hk) hk=f;
		if (this._rescache[hk]) return;
		this.toCache(f,hk);
		
		if (!opt) opt={};

		if (f.match(/\.js$/)) {
			//linking js
			$.ajax({
				async : false,
				type: "GET",
				url: f,
				dataType: "script"
			});
		}
		else
		if (f.match(/\.css$/)) {
			//linking css
			var lnk = document.createElement("link");		
			lnk.rel="stylesheet";
			lnk.type="text/css";
			lnk.media=opt.media||"screen";
			lnk.href=f;
			(document.getElementsByTagName("head"))[0].appendChild(lnk);
		}
	}

});

$.yii.toCache(' . $scriptList . ');

})(jQuery);
');

			return $html;
		}

		return '';
	}
	
	/**
	 * Inserts the scripts in the head section.
	 * @param string $output the output to be inserted with scripts.
	 */
	public function renderHead(&$output)
	{
		$html='';
		
		//nlac:!!!
		$html .= self::_renderBaseScriptFiles();
		
		foreach($this->metaTags as $meta)
			$html.=CHtml::metaTag($meta['content'],null,null,$meta)."\n";
		foreach($this->linkTags as $link)
			$html.=CHtml::linkTag(null,null,null,null,$link)."\n";
//		foreach($this->cssFiles as $url=>$media)
//			$html.=CHtml::cssFile($url,$media)."\n";
//		foreach($this->css as $css)
//			$html.=CHtml::css($css[0],$css[1])."\n";
		if($this->enableJavaScript)
		{
			if(isset($this->scriptFiles[self::POS_HEAD]))
			{
				foreach($this->scriptFiles[self::POS_HEAD] as $scriptFile)
					//$html.=CHtml::scriptFile($scriptFile)."\n";
					$html.=self::scriptFile($scriptFile)."\n";
			}

			if(isset($this->scripts[self::POS_HEAD]))
				$html.=CHtml::script(implode("\n",$this->scripts[self::POS_HEAD]))."\n";
		}
		
		//nlac: css moved to here
		foreach($this->cssFiles as $url=>$media)
			$html.=self::cssFile($url,$media)."\n";
		foreach($this->css as $css)
			$html.=CHtml::css($css[0],$css[1])."\n";

		if($html!=='')
		{
			$count=0;
			$output=preg_replace('/(<title\b[^>]*>|<\\/head\s*>)/is','<###head###>$1',$output,1,$count);
			if($count)
				$output=str_replace('<###head###>',$html,$output);
			else
				$output=$html.$output;
		}
	}

	/**
	 * Inserts the scripts at the beginning of the body section.
	 * @param string $output the output to be inserted with scripts.
	 */
	public function renderBodyBegin(&$output)
	{
		$html='';
		if(isset($this->scriptFiles[self::POS_BEGIN]))
		{
			foreach($this->scriptFiles[self::POS_BEGIN] as $scriptFile)
			//$html.=CHtml::scriptFile($scriptFile)."\n";
			$html.=self::scriptFile($scriptFile)."\n";
		}
		if(isset($this->scripts[self::POS_BEGIN]))
			$html.=CHtml::script(implode("\n",$this->scripts[self::POS_BEGIN]))."\n";

		if($html!=='')
		{
			$count=0;
			$output=preg_replace('/(<body\b[^>]*>)/is','$1<###begin###>',$output,1,$count);
			if($count)
				$output=str_replace('<###begin###>',$html,$output);
			else
				$output=$html.$output;
		}
	}

	/**
	 * Inserts the scripts at the end of the body section.
	 * @param string $output the output to be inserted with scripts.
	 */
	public function renderBodyEnd(&$output)
	{
		if(!isset($this->scriptFiles[self::POS_END]) && !isset($this->scripts[self::POS_END])
			&& !isset($this->scripts[self::POS_READY]) && !isset($this->scripts[self::POS_LOAD]))
			return;

		$fullPage=0;
		$output=preg_replace('/(<\\/body\s*>)/is','<###end###>$1',$output,1,$fullPage);
		$html='';
		if(isset($this->scriptFiles[self::POS_END]))
		{
			foreach($this->scriptFiles[self::POS_END] as $scriptFile)
				//$html.=CHtml::scriptFile($scriptFile)."\n";
				$html.=self::scriptFile($scriptFile)."\n";
		}
		$scripts=isset($this->scripts[self::POS_END]) ? $this->scripts[self::POS_END] : array();
		if(isset($this->scripts[self::POS_READY]))
		{
			if($fullPage)
				$scripts[]="jQuery(function($) {\n".implode("\n",$this->scripts[self::POS_READY])."\n});";
			else
				$scripts[]=implode("\n",$this->scripts[self::POS_READY]);
		}
		if(isset($this->scripts[self::POS_LOAD]))
		{
			if($fullPage)
				$scripts[]="jQuery(window).load(function() {\n".implode("\n",$this->scripts[self::POS_LOAD])."\n});";
			else
				$scripts[]=implode("\n",$this->scripts[self::POS_LOAD]);
		}
		if(!empty($scripts))
			$html.=CHtml::script(implode("\n",$scripts))."\n";

		if($fullPage)
			$output=str_replace('<###end###>',$html,$output);
		else
			$output=$output.$html;
	}

}
